# Ayam, a free 3D modeler for the RenderMan interface.
#
# Ayam is copyrighted 1998-2007 by Randolf Schultz
# (randolf.schultz@gmail.com) and others.
#
# All rights reserved.
#
# See the file License for details.

# objio.tcl - objio (Wavefront OBJ IO) plugin GUI code

uplevel #0 { array set objio_options {
WriteSelected 0
TessPoMesh 0
WriteHeader 0
Header "# Written by Ayam\n"
WriteCurves 1
WriteMaterials 1
MergeFaces 1
MergePVTags 1
ScaleFactor 1.0
RescaleKnots 0.0
filename ""
FileName "unnamed.obj"
Progress 0
CheckDegen 1
ReadSTrim 1
ReadMaterials 1
ReadTexCoords 1
UseMaterials 1
RationalStyle 0
DefaultIllum 0
header ""
}   }


# objio_export:
#  export scene to Wavefront OBJ format
#
proc objio_export { } {
    global ay ay_error tcl_platform objio_options aymainshortcuts
    set ay_error ""

    winAutoFocusOff

    set objio_options(oldfocus) [focus]

    if { $objio_options(filename) != "" } {
	set objio_options(FileName) $objio_options(filename)
    } else {
	if { $ay(filename) != "" &&\
		$objio_options(FileName) == "unnamed.obj" } {
	    set objio_options(FileName) [file rootname $ay(filename)].obj
	}
    }

    set w .objE
    set t "Export OBJ"
    winDialog $w $t

    set f [frame $w.f1]
    pack $f -in $w -side top -fill x

    set ay(bca) .objE.f2.bca
    set ay(bok) .objE.f2.bok

    set types { {"Wavefront OBJ" ".obj"}
	        {"Wavefront MTL" ".mtl"}
	        {"All files" *} }

    addSFileT $f objio_options FileName $types
    addCheckB $f objio_options WriteHeader [ms objio_options_WriteHeader]
    lappend l "# Written by Ayam $ay(ay_version)\n"
    lappend l "# Written by Ayam $ay(ay_version)\n# on %d for %u"
    addStringB $f objio_options Header [ms objio_options_Header] $l
    addCheckB $f objio_options WriteSelected [ms objio_options_Selected]
    addCheckB $f objio_options WriteCurves [ms objio_options_WriteCurves]
    addCheckB $f objio_options WriteMaterials [ms objio_options_WriteMaterials]
    addMenu $f objio_options DefaultIllum {"0 - Constant" "1 - Diffuse" "2 - Specular" "3 - Reflection" "4 - FakeGlass" "5 - FresnelReflect" "6 - Raytracing" "7 - Fresnel"}
    addCheckB $f objio_options TessPoMesh [ms objio_options_TessPoMesh]
    addParam $f objio_options ScaleFactor [list 0.01 0.1 1.0 10.0 100.0]
    addMenu $f objio_options RationalStyle {"Euclidean" "Homogeneous"}
    set objio_options(Progress) 0
    addProgress $f objio_options Progress

    set f [frame $w.f2]
    button $f.bok -text "Ok" -width 5 -command {
	global objio_options ay_error ayprefs

	# append extension
	if { [file extension $objio_options(FileName)] != ".mtl" } {
	    set objio_options(FileName) \
		[io_appext $objio_options(FileName) ".obj"]
	}

	set objio_options(filename) $objio_options(FileName)
	set objio_options(oldcd) [pwd]
	cd [file dirname $objio_options(FileName)]

	set objio_options(Cancel) 0
	update

	if { $objio_options(WriteHeader) } {
	    set objio_options(hmap) "%d"
	    lappend objio_options(hmap)\
		[clock format [clock seconds] -format {%Y-%m-%d}]

	    lappend objio_options(hmap) "%t"
	    lappend objio_options(hmap)\
		[clock format [clock seconds] -format {%H:%M}]

	    lappend objio_options(hmap) "%u"
	    lappend objio_options(hmap) $tcl_platform(user)
	    set objio_options(header)\
		[string map $objio_options(hmap) $objio_options(Header)]
	}

	ayError 4 "exportOBJ" \
	    "Starting export to: \"$objio_options(FileName)\"."

	if { [file extension $objio_options(FileName)] == ".mtl" } {
	    objioWrite [file tail $objio_options(FileName)]\
		-h $objio_options(WriteHeader)\
		-H $objio_options(header)\
		-s $objio_options(WriteSelected)\
		-i $objio_options(DefaultIllum)\
		-M 1
	} else {
	    objioWrite [file tail $objio_options(FileName)]\
		-h $objio_options(WriteHeader)\
		-H $objio_options(header)\
		-s $objio_options(WriteSelected)\
		-p $objio_options(TessPoMesh)\
		-c $objio_options(WriteCurves)\
		-m $objio_options(WriteMaterials)\
		-i $objio_options(DefaultIllum)\
		-f $objio_options(ScaleFactor)\
		-r $objio_options(RationalStyle)
	}

	if { ! $ayprefs(ExportSetsCD) } {
	    cd $objio_options(oldcd)
	}

	if { $ay_error < 1 } {
	    ayError 4 "exportOBJ" "Done."
	} else {
	    if { $ay_error > 1 } {
		ayError 2 "exportOBJ" "Failed."
	    }
	}

	grab release .objE
	restoreFocus $objio_options(oldfocus)
	destroy .objE
    }
    # ok button

    button $f.bca -text "Cancel" -width 5 -command "\
		global objio_options;\
                set objio_options(Cancel) 1;\
		grab release .objE;\
                restoreFocus $objio_options(oldfocus);\
		destroy .objE"

    pack $f.bok $f.bca -in $f -side left -fill x -expand yes
    pack $f -in $w -side bottom -fill x

    # Esc-key && close via window decoration == Cancel button
    bind $w <Escape> "$f.bca invoke"
    wm protocol $w WM_DELETE_WINDOW "$f.bca invoke"

    bind $w <Key-Return> "$::ay(bok) invoke;break"
    catch {bind $w <Key-KP_Enter> "$::ay(bok) invoke;break"}

    # establish "Help"-binding
    shortcut_addcshelp $w ayam-7.html expwav

    winRestoreOrCenter $w $t
    grab $w
    focus $w.f2.bok
    tkwait window $w

    winAutoFocusOn

    after idle viewMouseToCurrent

 return;
}
# objio_export


# objio_import:
#  import scene from the Wavefront OBJ format
#
proc objio_import { } {
    global ay ay_error objio_options aymainshortcuts
    set ay_error ""

    winAutoFocusOff

    set objio_options(oldfocus) [focus]

    if { $objio_options(filename) != "" } {
	set objio_options(FileName) $objio_options(filename)
    } else {
	set objio_options(FileName) "unnamed.obj"
    }

    set w .objI
    set t "Import OBJ"
    winDialog $w $t

    set f [frame $w.f1]
    pack $f -in $w -side top -fill x

    set ay(bca) .objI.f2.bca
    set ay(bok) .objI.f2.bok

    set types { {"Wavefront OBJ" ".obj"}
	        {"Wavefront MTL" ".mtl"}
                {"All files" *} }

    addFileT $f objio_options FileName $types
    addCheck $f objio_options MergeFaces
    addCheck $f objio_options MergePVTags
    addCheck $f objio_options ReadCurves
    addCheck $f objio_options ReadSTrim
    addMenu $f objio_options ReadTexCoords {"None" "Separately" "Combined"}
    addCheck $f objio_options ReadMaterials
    addCheck $f objio_options UseMaterials
    addParam $f objio_options RescaleKnots [list 0.0 1.0e-4]
    addParam $f objio_options ScaleFactor  [list 0.01 0.1 1.0 10.0 100.0]
    addCheck $f objio_options CheckDegen
    addMenu $f objio_options RationalStyle {"Euclidean" "Homogeneous"}
    addMenu $f objio_options DefaultIllum {"Auto" "0 - Constant" "1 - Diffuse" "2 - Specular" "3 - Reflection" "4 - FakeGlass" "5 - FresnelReflect" "6 - Raytracing" "7 - Fresnel"}
    set objio_options(Progress) 0
    addProgress $f objio_options Progress

    set f [frame $w.f2]
    button $f.bok -text "Ok" -width 5 -command {
	global objio_options ay_error ayprefs

	if { ! [file readable $objio_options(FileName)] } {
	    ayError 10 "importOBJ" $objio_options(FileName)
	    break;
	}

	set objio_options(filename) $objio_options(FileName)
	set objio_options(oldcd) [pwd]
	cd [file dirname $objio_options(FileName)]

	set objio_options(Cancel) 0
	update
	set illum [expr $objio_options(DefaultIllum) - 1]

	ayError 4 "importOBJ" \
		"Starting import of: \"$objio_options(FileName)\"."

	objioRead [file tail $objio_options(FileName)]\
	    -m $objio_options(MergeFaces)\
	    -p $objio_options(MergePVTags)\
	    -k $objio_options(RescaleKnots)\
	    -f $objio_options(ScaleFactor)\
	    -d $objio_options(CheckDegen)\
	    -ra $objio_options(RationalStyle)\
	    -rc $objio_options(ReadCurves)\
	    -rm $objio_options(ReadMaterials)\
	    -rs $objio_options(ReadSTrim)\
	    -rt $objio_options(ReadTexCoords)\
	    -um $objio_options(UseMaterials)\
	    -i $illum

	if { $ay_error < 1 } {
	    ayError 4 "importOBJ" "Done."
	} else {
	    if { $ay_error > 1 } {
		ayError 2 "importOBJ" "Failed."
	    } else {
		ayError 4 "importOBJ" "Finished with warnings."
	    }
	}

	if { ! $ayprefs(ImportSetsCD) } {
	    cd $objio_options(oldcd)
	}

	goTop
	selOb
	set ay(CurrentLevel) "root"
	set ay(SelectedLevel) "root"
	update

	uS
	rV

	set ay(sc) 1

	grab release .objI
	restoreFocus $objio_options(oldfocus)
	destroy .objI
    }
    # ok button

    button $f.bca -text "Cancel" -width 5 -command "\
                global objio_options;\
                set objio_options(Cancel) 1;\
		grab release .objI;\
                restoreFocus $objio_options(oldfocus);\
		destroy .objI"

    pack $f.bok $f.bca -in $f -side left -fill x -expand yes
    pack $f -in $w -side bottom -fill x

    # Esc-key && close via window decoration == Cancel button
    bind $w <Escape> "$f.bca invoke"
    wm protocol $w WM_DELETE_WINDOW "$f.bca invoke"

    bind $w <Key-Return> "$::ay(bok) invoke;break"
    catch {bind $w <Key-KP_Enter> "$::ay(bok) invoke;break"}

    # establish "Help"-binding
    shortcut_addcshelp $w ayam-7.html impwav

    winRestoreOrCenter $w $t
    grab $w
    focus $w.f2.bok
    tkwait window $w

    winAutoFocusOn

    after idle viewMouseToCurrent

 return;
}
# objio_import


global ay

$ay(im) add command -label "Wavefront OBJ" -command "objio_import"
$ay(em) add command -label "Wavefront OBJ" -command "objio_export"
