/* RSmtld.sl - Wavefront MTL alike displacement shader
 * This shader is part of Ayam, a free 3D modeler for the RenderMan interface.
 * Copyright (c) 1998-2023 by Randolf Schultz (randolf.schultz@gmail.com)
 * All rights reserved. See License for details.
 * Parameters:
 * disp_map - displace point to render along normal using a scalar value
 *            read from a texture map file
 * base - offset to apply to the texture map value
 * gain - multiplier to apply to the offset texture map value
 */
displacement
RSmtld(string disp_map = ""; float base = 0.0, gain = 1.0;)
{
  vector Ns = vtransform("shader", N);
  point Ps = transform("shader", P);
  float magnitude = (base + float texture(disp_map)) * gain;
  Ps += magnitude * normalize(Ns);
  P = transform("shader", "current", Ps);
  N = calculatenormal(P);
}
